@@ -25,7 +25,15 @@ use XML::Smart::Data qw(data) ;
 use XML::Smart::XPath qw(xpath XPath xpath_pointer XPath_pointer) ;
 
 our ($VERSION) ;
-$VERSION = '1.4' ;
+$VERSION = '1.5' ;
+
+#################
+# NO_XML_PARSER #
+#################
+
+sub NO_XML_PARSER {
+  $XML::Smart::Tree::NO_XML_PARSER = !@_ ? 1 : ( $_[0] ? 1 : undef ) ;
+}
 
 #######
 # NEW #
@@ -176,6 +184,8 @@ sub base {
   code      => \&find_arg , 
   ) ;
   
+  bless($base,__PACKAGE__) ;
+  
   $$base->{tree} = $this->tree ;
   $$base->{point} = $$base->{tree} ;
   
@@ -257,6 +267,18 @@ sub path_as_xpath {
   return $path ;
 }
 
+########
+# ROOT #
+########
+
+sub root {
+  my $this = shift ;
+  
+  my $root = ( $this->base->nodes_keys )[0] ;
+
+  return $root ;
+}
+
 #######
 # KEY #
 #######
@@ -351,6 +373,7 @@ sub _copy_hash {
 ########
 
 sub tree { return( ${$_[0]}->{tree} ) ;}
+sub tree_pointer { &pointer ;}
 
 ###########
 # POINTER #
@@ -368,9 +391,11 @@ sub pointer {
 sub cut_root {
   my $this = shift ;
 
-  if (keys %{$this} > 1) { return $this ;}
+  my @nodes = $this->nodes_keys ;
+
+  if ($#nodes > 0) { return $this ;}
   
-  my $root = (keys %{$this})[0] ;
+  my $root = @nodes[0] ;
   return( $this->{$root} ) ;
 }
 
@@ -395,11 +420,13 @@ sub is_node {
 sub args {
   my $this = shift ;
   my @args ;
-  my $nodes = $this->back->{'/nodes'} ; 
+
+  my $nodes = $this->back->{'/nodes'} ;
+  my $pointer = $$this->{point} ;
   
   foreach my $Key ( keys %$this ) {
     if ( !$$nodes{$Key} ) {
-      if ( (!ref $$this{$Key}) || (ref($$this{$Key}) eq 'HASH') || (ref($$this{$Key}) eq 'ARRAY' && $#{$$this{$Key}} == 0) ) {
+      if ( (!ref $$pointer{$Key}) || (ref($$pointer{$Key}) eq 'HASH') || (ref($$pointer{$Key}) eq 'ARRAY' && $#{$$pointer{$Key}} == 0) ) {
         push(@args , $Key) ;
       }
     }
@@ -433,13 +460,14 @@ sub nodes {
   my $this = shift ;
 
   my $nodes = $this->{'/nodes'}->pointer ;
+  my $pointer = $$this->{point} ;
   
   my @nodes ;
   
   foreach my $Key ( keys %$this ) {
-    if ( $$nodes{$Key} || (ref($$this{$Key}) eq 'HASH') || (ref($$this{$Key}) eq 'ARRAY' && $#{$$this{$Key}} > 0)  ) {
-      if (ref($$this{$Key}) eq 'ARRAY') {
-        my $n = $#{$$this{$Key}} ;
+    if ( $$nodes{$Key} || (ref($$pointer{$Key}) eq 'HASH') || (ref($$pointer{$Key}) eq 'ARRAY' && $#{$$pointer{$Key}} > 0)  ) {
+      if (ref($$pointer{$Key}) eq 'ARRAY') {
+        my $n = $#{$$pointer{$Key}} ;
         for my $i (0..$n) {
           push(@nodes , $this->{$Key}[$i]) ;
         }
@@ -461,10 +489,12 @@ sub nodes_keys {
   my $this = shift ;
 
   my $nodes = $this->{'/nodes'}->pointer ;
+  my $pointer = $$this->{point} ;
+  
   my @nodes ;
   
   foreach my $Key ( keys %$this ) {
-    if ( $$nodes{$Key} || (ref($$this{$Key}) eq 'HASH') || (ref($$this{$Key}) eq 'ARRAY' && $#{$$this{$Key}} > 0)  ) {
+    if ( $$nodes{$Key} || (ref($$pointer{$Key}) eq 'HASH') || (ref($$pointer{$Key}) eq 'ARRAY' && $#{$$pointer{$Key}} > 0)  ) {
       push(@nodes , $Key) ;
     }
   }
@@ -482,7 +512,11 @@ sub set_node {
   if ( !@_ ) { $bool = 1 ;}
   
   my $key = $this->key ;
-  my $nodes = $this->{'/nodes'}->pointer ;
+  
+  my $back = $this->back ;
+  
+  $back->{'/nodes'} = {} if $back->{'/nodes'}->null ;
+  my $nodes = $back->{'/nodes'}->pointer ;
   
   if ( $bool ) {
     $$nodes{$key} = 1 ;
@@ -548,6 +582,8 @@ sub ret {
     elsif ($type =~ /\%$/) { @ret = %{$back->{$key}[$i]} ;}
   }
   else {
+    if ( $this->null ) { return ;}
+    
     if    ($type =~ /\$$/) { @ret = $this->content ;}
     elsif ($type =~ /\@$/) { @ret = @{$this} ;}
     elsif ($type =~ /\%$/) { @ret = %{$this} ;}
@@ -693,8 +729,10 @@ sub find_arg {
 
 sub content {
   my $this = shift ;
+  my $set_i = shift if $#_ > 0 ;
   
   if ( defined $$this->{content} ) {
+    if (@_) { ${$$this->{content}} = $_[0] ;}
     return ${$$this->{content}} ;
   }
   
@@ -705,9 +743,12 @@ sub content {
   
   if (ref($$this->{point}{$key}) eq 'ARRAY') {
     if ($i eq '') { $i = 0 ;}
+    if (@_) { $$this->{point}{$key}[$i] = $_[0] ;}
     return $$this->{point}{$key}[$i] ;
   }
-  elsif (defined $$this->{point}{$key}) {
+  elsif (exists $$this->{point}{$key}) {
+    if (@_ && ( my $tie = tied($$this->{point}{$key}) )) { $tie->STORE($set_i , $_[0]) ;}
+    if ( wantarray && ( my $tie = tied($$this->{point}{$key}) ) ) { return $tie->FETCH(1) ;}
     return $$this->{point}{$key} ;
   }
   
@@ -885,7 +926,7 @@ I<XML::Smart::HTMLParser> can be used to load/parse wild/bad XML data, or HTML t
 
 =head1 METHODS
 
-=head2 new (FILE|DATA|URL , PARSER)
+=head2 new (FILE|DATA|URL , PARSER , OPTIONS)
 
 Create a XML object.
 
@@ -930,16 +971,80 @@ If not set it will look for XML::Parser and load it.
 If XML::Parser can't be loaded it will use XML::Smart::Parser, that actually is a
 clone of XML::Parser::Lite with some fixes.
 
+=item OPTIONS
+
+You can force the uper case and lower case for tags (nodes) and arguments (attributes).
+
+=over 10
+
+=item lowtag
+
+Make the tags lower case.
+
+=item lowarg
+
+Make the arguments lower case.
+
+=item upertag
+
+Make the tags uper case.
+
+=item uperarg
+
+Make the arguments uper case.
+
+=item on_start (CODE) I<*optional>
+
+Code/sub to call on start a tag.
+
+I<** This will be called after XML::Smart parse the tag, should be used only if you want to change the tree.>
+
+=item on_char (CODE) I<*optional>
+
+Code/sub to call on content.
+
+I<** This will be called after XML::Smart parse the tag, should be used only if you want to change the tree.>
+
+=item on_end (CODE) I<*optional>
+
+Code/sub to call on end a tag.
+
+I<** This will be called after XML::Smart parse the tag, should be used only if you want to change the tree.>
+
 =back
 
-=head2 copy()
+I<** This options are applied when the XML data is loaded. For XML generation see data() OPTIONS.>
 
-Return a copy of the XML::Smart object (pointing to the base).
+=back
 
-** This is good when you want to keep 2 versions of the same XML tree in the memory,
-since one object can't change the tree of the other!
+Example of use:
 
-=head2 base()
+  my $xml = XML::Smart->new($data , 'html' ,
+  lowtag => 1 ,
+  lowarg => 1 ,
+  on_char => sub {
+               my ( $tag , $pointer , $pointer_back , $cont) = @_ ;
+               $pointer->{extra_arg} = 123 ; ## add an extrar argument.
+               $pointer_back->{$tag}{extra_arg} = 123 ; ## Same, but using the previous pointer.
+               $$cont .= "\n" ; ## append data to the content.
+             }
+  ) ;
+
+=head2  args()
+
+Return the arguments names (not nodes).
+
+=head2  args_values()
+
+Return the arguments values (not nodes).
+
+=head2  back()
+
+Get back one level the pointer in the tree.
+
+** Se I<base()>.
+
+=head2  base()
 
 Get back to the base of the tree.
 
@@ -952,57 +1057,7 @@ again (an object that points to the base):
   my $XML2 = $srv->base() ;
   $XML2->{root}{hosts}...
 
-=head2 back()
-
-Get back one level the pointer in the tree.
-
-** Se I<base()>.
-
-=head2 cut_root()
-
-Cut the root key:
-
-  my $srv = $XML->{rootx}{host}{server} ;
-  
-  ## Or if you don't know the root name:
-  $XML = $XML->cut_root() ;
-  my $srv = $XML->{host}{server} ;
-
-** Note that this will cut the root of the pointer in the tree.
-So, if you are in some place that have more than one key (multiple roots), the
-same object will be retuned without cut anything.
-
-=head2 key()
-
-Return the key of the value.
-
-If wantarray return the index too: return(KEY , I) ;
-
-=head2 i()
-
-Return the index of the value.
-
-** If the value is from an hash key (not an ARRAY ref) undef is returned.
-
-=head2 set_node(BOOL)
-
-Set/unset the current key as a node (tag).
-
-** If BOOL is not defined will use I<true>.
-
-=head2 set_tag
-
-Same as set_node.
-
-=head2 set_order(KEYS)
-
-Set the order of the keys (nodes and attributes) in this point.
-
-=head2 null()
-
-Return I<true> if the XML object has a null tree or if the pointer is in some place that doesn't exist.
-
-=head2 content()
+=head2  content()
 
 Return the content of a node:
 
@@ -1017,68 +1072,78 @@ Return the content of a node:
   ## or just:
   my $content = $XML->{foo} ;
 
-=head2 args()
+B<Also can be used with multiple contents:>
 
-Return the arguments names (not nodes).
+For this XML data:
 
-=head2 args_values()
+  <root>
+  content0
+  <tag1 arg="1"/>
+  content1
+  </root>
 
-Return the arguments values (not nodes).
+Getting all the content:
 
-=head2 nodes()
+  my $all_content = $XML->{root}->content ;
+  print "[$all_content]\n" ;
 
-Return the nodes (objects) in the pointer (keys that aren't arguments).
+Output:
 
-=head2 nodes_keys()
-
-Return the nodes names (not the object) in the pointer (keys that aren't arguments).
-
-=head2 path()
-
-Return the path of the pointer.
-
-I<Example>:
-
-  /hosts/server[1]/address[0]
-
-B<Note that the index is 0 based and 'address' can be an attribute or a node, what is not compatible with XPath.>
-
-B<** See I<path_as_xpath()>.>
-
-=head2 path_as_xpath()
-
-Return the path of the pointer in the XPath format.
-
-=head2 xpath() || XPath()
-
-Return a XML::XPath object, based in the XML root in the tree.
-
-  ## look from the root:
-  my $data = $XML->XPath->findnodes_as_string('/') ;
-
-I<** Need XML::XPath installed, but only load when is needed.>
-
-=head2 xpath_pointer() || XPath_pointer() 
-
-Return a XML::XPath object, based in the XML::Smart pointer in the tree.
-
-  ## look from this point, soo XPath '/' actually starts at /server/:
+  [
+  content0
   
-  my $srvs = $XML->{server} ;
-  my $data = $srvs->XPath_pointer->findnodes_as_string('/') ;
+  content1
+  ]
 
-I<** Need XML::XPath installed, but only load when is needed.>
+Getting in parts:
 
-=head2 tree()
+  my @contents = $XML->{root}->content ;
+  print "[@contents[0]]\n" ;
+  print "[@contents[1]]\n" ;
 
-Return the HASH tree of the XML data.
+Output
 
-** Note that the real HASH tree is returned here. All the other ways return an
-object that works like a HASH/ARRAY through tie.
+  [
+  content0
+  ]
+  [
+  content1
+  ]
 
-=head2 pointer
+B<Setting multiple contents:>
 
-Return the HASH tree from the pointer.
+  $XML->{root}->content(0,"aaaaa") ;
+  $XML->{root}->content(1,"bbbbb") ;
+
+Output now will be:
+
+  [aaaaa]
+  [bbbbb]
+
+And now the XML data generated will be:
+
+  <root>aaaaa<tag1 arg="1"/>bbbbb</root>
+
+=head2  copy()
+
+Return a copy of the XML::Smart object (pointing to the base).
+
+** This is good when you want to keep 2 versions of the same XML tree in the memory,
+since one object can't change the tree of the other!
+
+=head2  cut_root()
+
+Cut the root key:
+
+  my $srv = $XML->{rootx}{host}{server} ;
+  
+  ## Or if you don't know the root name:
+  $XML = $XML->cut_root() ;
+  my $srv = $XML->{host}{server} ;
+
+** Note that this will cut the root of the pointer in the tree.
+So, if you are in some place that have more than one key (multiple roots), the
+same object will be retuned without cut anything.
 
 =head2 data (OPTIONS)
 
@@ -1186,12 +1251,113 @@ Make the tree from current point in the XML tree (not from the base as data()).
 
 Accept the same OPTIONS of the method B<I<data()>>.
 
+
+=head2  i()
+
+Return the index of the value.
+
+** If the value is from an hash key (not an ARRAY ref) undef is returned.
+
+=head2  is_node()
+
+Return if a key is a node.
+
+=head2  key()
+
+Return the key of the value.
+
+If wantarray return the index too: return(KEY , I) ;
+
+=head2  nodes()
+
+Return the nodes (objects) in the pointer (keys that aren't arguments).
+
+=head2  nodes_keys()
+
+Return the nodes names (not the object) in the pointer (keys that aren't arguments).
+
+=head2  null()
+
+Return I<true> if the XML object has a null tree or if the pointer is in some place that doesn't exist.
+
+=head2  path()
+
+Return the path of the pointer.
+
+I<Example>:
+
+  /hosts/server[1]/address[0]
+
+B<Note that the index is 0 based and 'address' can be an attribute or a node, what is not compatible with XPath.>
+
+B<** See I<path_as_xpath()>.>
+
+=head2  path_as_xpath()
+
+Return the path of the pointer in the XPath format.
+
+=head2  pointer
+
+Return the HASH tree from the pointer.
+
+=head2 root
+
+Return the ROOT name of the XML tree (main key).
+
+** See also I<key()> for sub nodes.
+
 =head2 save (FILEPATH , OPTIONS)
 
 Save the XML data inside a file.
 
 Accept the same OPTIONS of the method B<I<data()>>.
 
+=head2  set_node(BOOL)
+
+Set/unset the current key as a node (tag).
+
+** If BOOL is not defined will use I<true>.
+
+=head2  set_order(KEYS)
+
+Set the order of the keys (nodes and attributes) in this point.
+
+=head2  set_tag
+
+Same as set_node.
+
+=head2  tree()
+
+Return the HASH tree of the XML data.
+
+** Note that the real HASH tree is returned here. All the other ways return an
+object that works like a HASH/ARRAY through tie.
+
+=head2  tree_pointer()
+
+Same as I<pointer()>.
+
+=head2  xpath() || XPath()
+
+Return a XML::XPath object, based in the XML root in the tree.
+
+  ## look from the root:
+  my $data = $XML->XPath->findnodes_as_string('/') ;
+
+I<** Need XML::XPath installed, but only load when is needed.>
+
+=head2  xpath_pointer() || XPath_pointer() 
+
+Return a XML::XPath object, based in the XML::Smart pointer in the tree.
+
+  ## look from this point, soo XPath '/' actually starts at /server/:
+  
+  my $srvs = $XML->{server} ;
+  my $data = $srvs->XPath_pointer->findnodes_as_string('/') ;
+
+I<** Need XML::XPath installed, but only load when is needed.>
+
+
 =head1 ACCESS
 
 To access the data you use the object in a way similar to HASH and ARRAY:
